var ASMAPI = Java.type("net.minecraftforge.coremod.api.ASMAPI");
var Opcodes = Java.type('org.objectweb.asm.Opcodes');
var InsnList = Java.type('org.objectweb.asm.tree.InsnList');
var InsnNode = Java.type('org.objectweb.asm.tree.InsnNode');
var MethodInsnNode = Java.type('org.objectweb.asm.tree.MethodInsnNode');
var VarInsnNode = Java.type('org.objectweb.asm.tree.VarInsnNode');


var METHOD_INIT = ASMAPI.mapMethod("<init>");//init
var METHOD_INIT_DESC = "(FFII)V";

var METHOD_RENDER = ASMAPI.mapMethod("func_78088_a");//render
var METHOD_RENDER_DESC = "(Lnet/minecraft/entity/LivingEntity;FFFFFF)V";

var METHOD_ROTATE = ASMAPI.mapMethod("func_212844_a_");//setRotationAngles
var METHOD_ROTATE_DESC = "(Lnet/minecraft/entity/LivingEntity;FFFFFF)V";

var NUM_FOUND = 0;


function log(message)
{
    print("[GenderCore-ModelBiped]: " + message);
}

function initializeCoreMod()
{
	return {
		"GenderCore-ModelBiped": {
			"target": {
				"type": "CLASS",
                'name': 'net.minecraft.client.renderer.entity.model.BipedModel'
			},
			"transformer": function(classNode) {                									
                for each (var method in classNode.methods) {					
                    if (method.name.equals(METHOD_INIT) && method.desc.equals(METHOD_INIT_DESC)) {
                        log("Transforming " + classNode.name+"."+method.name);
    					NUM_FOUND++;
                        doPatchInit(method);
                    } else if (method.name.equals(METHOD_RENDER) && method.desc.equals(METHOD_RENDER_DESC)) {
                        log("Transforming " + classNode.name+"."+method.name);
    					NUM_FOUND++;
                        doPatchRender(method);
                    } else if (method.name.equals(METHOD_ROTATE) && method.desc.equals(METHOD_ROTATE_DESC)) {
                        log("Transforming " + classNode.name+"."+method.name);
    					NUM_FOUND++;
                        doPatchRotate(method);
                    } //else log("Did not match method DESC " + method.name + method.desc);
                }
                log("Found methods: "+NUM_FOUND+" of 3");
				return classNode;
			}
		}
	};
}

function findFinalReturn(method)
{
	var finalReturn;
    for (var i = method.instructions.size() - 1; i >= 0; i--) {
		var instruction = method.instructions.get(i);
		var instructionBefore = method.instructions.get(i);		
		//log("Looking for last return... " + method.name+method.desc+" "+instruction.getOpcode().toString());				
		if (method.instructions.get(i).getOpcode() == Opcodes.RETURN) {
			finalReturn = instruction;
			//log("Found last return " + method.name+method.desc+" "+instruction.getOpcode().toString());
			break;
		}
	}	
	if (finalReturn == null) {
		log("Failed to find last return " + method.name+method.desc);
	}
	return finalReturn;
}

function doPatchInit(method)
{
    var finalReturn = findFinalReturn(method);    
	if(finalReturn != null) {
	    var toInject = new InsnList();
	    toInject.add(new VarInsnNode(Opcodes.ALOAD, 0));
		toInject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, "com/github/ipixeli/Gender/sides/client/render/RenderHooks", "modelBiped_init", "(Lnet/minecraft/client/renderer/entity/model/BipedModel;)V"));    
	    method.instructions.insertBefore(finalReturn, toInject);
    }
}

function doPatchRender(method)
{
    var finalReturn = findFinalReturn(method);    
	if(finalReturn != null) {
	    var toInject = new InsnList();
	    toInject.add(new VarInsnNode(Opcodes.ALOAD, 0));
	    toInject.add(new VarInsnNode(Opcodes.ALOAD, 1));
	    toInject.add(new VarInsnNode(Opcodes.FLOAD, 7));
		toInject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, "com/github/ipixeli/Gender/sides/client/render/RenderHooks", "modelBiped_render", "(Lnet/minecraft/client/renderer/entity/model/BipedModel;Lnet/minecraft/entity/Entity;F)V"));
	    method.instructions.insertBefore(finalReturn, toInject);
    }
}

function doPatchRotate(method)
{
    var finalReturn = findFinalReturn(method);    
	if(finalReturn != null) {
	    var toInject = new InsnList();
	    toInject.add(new VarInsnNode(Opcodes.ALOAD, 0));
		toInject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, "com/github/ipixeli/Gender/sides/client/render/RenderHooks", "modelBiped_setRotations", "(Lnet/minecraft/client/renderer/entity/model/BipedModel;)V"));
	    method.instructions.insertBefore(finalReturn, toInject);
    }
}